anim8 = require "anim8"

require "camera"
require "player/player"
require "titleScreen"
require "pauseScreen"
require "backgroundTerrain"
require "spawns"
require "NPCs/villager"
require "NPCs/innkeeper"
require "enemies/skeleton/skeleton"
require "enemies/wolf/wolf"
require "enemies/scarecrow/scarecrow"
require "enemies/bat/bat"
require "dialogs"
require "mapConnector"
require "objects/small-heart"

gameState = "titleScreen"
scale = 2
windowWidth		= 400
windowHeight	= 300

function love.load()
	load_resources()
end

function love.update(dt)
	--[[
	Po ostatnim hicie player opada na ziemię - timer leci kiedy player.isDying
	Jeśli player.isDying i timer = 0 --> gameState = "gameOverScreen" --> Czarny ekran + pojawiający się YOU ARE DEAD
	Masz zapisane X, Y, nazwę mapy, na której umarłeś i na której się zrespawnujesz
	Press Enter to continue
	gameOverScreen fade out z timerem i po timerze gameState = playingm
	]]
	if dt > 0.05 then dt = 0.005 end

	-- if player is dead slow the game
	if player.status == "dead" then dt = dt / 2 end
	if gameState == "playing" then
		-- move the background
		updateBackground(dt, player.x)
		-- reset player x/y
		if love.keyboard.isDown( "z" ) then
			player:spawnPlayer(200, 456)
		end
		-- if something goes REALLY wrong, restart player Y coord (USELESS 8D)
		if player.y > 1600 then player.y = 898 end
		-- update player position
		player:update(dt, gravity, currentMap)
		-- update entities
		for k, entity in pairs( entities ) do
			entity:update(dt, k)
		end
		-- update dialog boxes
		updateDialog(dt)
		-- update level (spawn entities)
		updateSpawns(dt, currentMap)
		-- move player to different locations
		updateMapConnector(dt, player.x, player.y, currentPlace, currentMap)

		-- move the camera
		updateCamera(dt)
	end
	if gameState == "mapTransition" then
		updateMapTransition(dt)
	end
end

function love.draw()
	love.graphics.scale(scale, scale)
	if gameState == "titleScreen" then
		drawTitleScreen()
	elseif gameState == "playing" then
		camera:set()

		drawBackground()
		local drawRangeX, drawRangeY
		if camX + windowWidth < currentMap.width * currentMap.tileWidth then
			drawRangeX = math.floor(camX)
		else
			drawRangeX = 0
		end
		if camY + windowHeight < currentMap.height * currentMap.tileHeight then
			drawRangeY = math.floor(camY)
		else
			drawRangeY = 0
		end
		currentMap:setDrawRange(drawRangeX, drawRangeY, windowWidth, windowHeight + 32)
		currentMap:draw()

		for k, entity in pairs( entities ) do
			entity:draw()
		end
		
		player:drawPlayer()

		drawMapConnector()

		drawDialog()
		if spawnX ~= nil and spawnY ~= nil then love.graphics.rectangle("fill", spawnX, spawnY, 2, 2) end

		camera:unset()

		drawHUD()

		if player.hasJumped then love.graphics.print("hasJumped = true", 5, 20) else love.graphics.print("hasJumped = false", 5, 20) end
		if player.onFloor then love.graphics.print("onFloor = true", 5, 40) else love.graphics.print("onFloor = false", 5, 40) end
		if player.isJumping then love.graphics.print("isJumping = true", 5, 60) else love.graphics.print("isJumping = false", 5, 60) end
		if player.onStairs then love.graphics.print("onStairs = true", 5, 80) else love.graphics.print("onStairs = false", 5, 80) end
		love.graphics.print("x = "..player.x, 5, 100)
		love.graphics.print("y = "..player.y, 5, 115)
		love.graphics.print("invulneratbleTimer ="..player.invulnerableTime, 5, 130)

		--love.graphics.print("Spawns = "..#spawns, 5, 120)
		--love.graphics.print("Tile = "..math.ceil(player.x / 16), 5, 100)
		--if spawns[1] ~= nil then love.graphics.print(spawns[1].status, 5, 140) end
		--if entities[1] ~= nil then love.graphics.print("state:        "..entities[1].state, 5, 25) end
		--if entities[1] ~= nil then love.graphics.print("x difference: "..entities[1].x - player.x, 5, 50) end
		--if entities[1] ~= nil then love.graphics.print("Destination:  "..entities[1].patternTimer, 5, 75) end
		--if entities[1] ~= nil then love.graphics.print("Gowno:        "..entities[1].x, 5, 100) end
		--spawnX = camX - 32
		--spawnX = camX + 332
		love.graphics.rectangle("fill", camX - 32, player.y, 1, 300)
		love.graphics.rectangle("fill", camX + 332, player.y, 1, 300)
		

	elseif gameState == "paused" then
		drawPauseScreen()
	elseif gameState == "gameOverScreen" then
		love.graphics.print('G A M E  O V E R', windowWidth / 2 - 20, windowHeight / 2)
	end
end

function drawHUD()
	love.graphics.draw(hudHead, 9, 9)
	for i=1, player.maxHealth, 1 do
		if i <= player.health then
			love.graphics.draw(hudHealthFull, 40 + (i-1) * 6, 16)
		else
			love.graphics.draw(hudHealthEmpty, 40 + (i-1) * 6, 16)
		end
	end
	love.graphics.draw(hudHearts, 320, 16)
	love.graphics.setColor(158,0,2)
	love.graphics.printf(player.hearts, 334, 20, 30, "right")
	love.graphics.setColor(255,255,255)
end

function load_resources()
	debug = 0
	love.window.setMode(windowWidth * scale, windowHeight * scale)
	-- for full screen
	--love.window.setMode(windowWidth * scale, windowHeight * scale, {fullscreen=true})

	unblurredFont = love.graphics.newImage("gfx/font.png")
	font = love.graphics.newImageFont(unblurredFont,
		" abcdefghijklmnopqrstuvwxyz" ..
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ0" ..
		"123456789.,!?-+/():;%&`'*#=[]\"")
	love.graphics.setFont(font)

	-- HUD
	hudHead = love.graphics.newImage("gfx/hud-head.png")
	hudHealthFull = love.graphics.newImage("gfx/hud-health-full.png")
	hudHealthEmpty = love.graphics.newImage("gfx/hud-health-empty.png")
	hudHearts = love.graphics.newImage("gfx/hud-hearts.png")

	-- Load music and sounds
	loadSound()

	-- Load maps & stuff
	local ATL = require("AdvTiledLoader")
	ATL.Loader.path = 'maps/'

	map01 = ATL.Loader.load('map01.tmx')
	map02 = ATL.Loader.load('map02.tmx')
	map03 = ATL.Loader.load('map03.tmx')
	map04 = ATL.Loader.load('map04.tmx')

	currentMap = map02
	currentPlace = "Arefu Fields"
	lastSpawnPosition = {1000,456} -- for respawning after death

	love.graphics.setBackgroundColor(71,31,116)

	camera:setBounds( 0, 0, (currentMap.width * currentMap.tileWidth) - windowWidth + 1, currentMap.height * currentMap.tileHeight - windowHeight )
	camX = 1000
	camY = 456

	player = Player:new()

	player.x = 1000
	player.y = 0
	player.width = 32
	player.height = 64
	player.walkSpeed = 125
	player.jumpSpeed = -320
	hasJumped = false

	gravity = 750

	-- Load pause screen images
	loadPauseScreen()

	-- Load dialog images
	loadDialog()

	-- Load start map spawns | uses images from loadDialog()
	entities = {} -- Array for every spawned NPC/enemy/bullet
	spawnPoints = loadSpawnPoints(currentPlace)

	-- Load backgroundTerrain gfx
	loadBackgroundAssets()
	loadBackgroundTable(currentPlace)
end

function loadSound()
	sfxWhipHit = love.audio.newSource("sfx/whipHit.mp3", "static")
	sfxWhipUse = love.audio.newSource("sfx/whipUse.mp3", "static")
	sfxHitGround = love.audio.newSource("sfx/hitGround.mp3", "static")
	sfxDamageTaken = love.audio.newSource("sfx/damageTaken.mp3", "static")
	sfxPlayerDeath = love.audio.newSource("sfx/playerDeath.mp3", "static")
	sfxEnemyDeath = love.audio.newSource("sfx/enemyDeath.mp3", "static")
	sfxConfirm = love.audio.newSource("sfx/confirm.mp3", "static")
end

function love.keypressed(key, unicode)
	if player.status == "alive" then player:keypressed(key, unicode) end
	-- keypressed for every entity
	for k, entity in pairs( entities ) do
		entity:keypressed(key, unicode)
	end

	-- keypressed on dialogs
	dialogKeypressed(key, unicode)
	-- keypressed on pause screen
	pauseScreenKeypressed(key, unicode)

	-- keypressed on teleporting (doors)
	mapConnectorKeypressed(key, unicode)

	-- ENTER
	if key == "return" then
		if gameState == "paused" then
			gameState = "playing"
			pauseScreenState = "overview"
			pauseScreenPosition = 1
		elseif gameState == "titleScreen" then
			player:spawnPlayer(300, 456)
			gameState = "playing"
		elseif gameState == "playing" then
			gameState = "paused"
		elseif gameState == "gameOverScreen" then
			gameState = "titleScreen"
			-- put here spawning player in last position, cuting hearts in half and so on
			player.status = "alive"
			player.health = 1
			player:spawnPlayer(1000, 456)
		end
	end
	if key == ("escape") then
		if gameState == "playing" then
			gameState = "paused" 
		elseif gameState == "paused" then
			gameState = "playing"
		elseif gameState == "titleScreen" then
			love.event.quit()
		end
	end
	-- key for title screen Try to figure out how to go to title screen
	if key == ("]") then gameState = "titleScreen" end
end

function love.keyreleased(key)
	-- stop the player if movement key is realeased ("if" is to prevent reseting animation)
	if (key == "d" or key == "a" or key == "w" or key == "s") and player.onFloor and player.status == "alive" then
		player:stop()
		if (not love.keyboard.isDown( "a" )) and (not love.keyboard.isDown( "d" )) then
			playerAnimation:gotoFrame(1)
		end
	end
	-- reset animation when you climb stairs
	if key == "w" and player.onStairs then
		animWalkUpstairsRight:gotoFrame(7)
		animWalkUpstairsLeft:gotoFrame(7)
	end
	if key == "s" and player.onStairs then
		animWalkDownstairsRight:gotoFrame(1)
		animWalkDownstairsLeft:gotoFrame(1)
	end
	if (key == "w" or key == "s") and player.onStairs then
		playerAnimation:pause();
	end

	-- spawn heart
	if key == "f" then
		--player.hasJumped = false
		player:jump()
		local sx = player.x + 40
		local sy = player.y
		table.insert( entities, smallHeart:new(sx,sy))
	end
end

function love.focus(f)
	if not f and gameState == "playing" then
		gameState = "paused"
	end
end

function math.clamp(x, min, max)
	return x < min and min or (x > max and max or x)
end

function math.round(x, z)
	return math.floor(x + z)
end

function isColliding(map, x, y)
	local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
	local tile = map("walls")(tileX, tileY)

	return not(tile == nil)
end