Player = {}

local playerImage = love.graphics.newImage("player/sprite.png")
local playerImage2 = love.graphics.newImage("player/spriteAttack.png")

-- preparing animation
local g = anim8.newGrid(32, 64, playerImage:getWidth(), playerImage:getHeight())

animWalkLeft	= anim8.newAnimation('loop', g('1-13,2'), 0.1, {[4]=0.12,[5]=0.14,[6]=0.14,[7]=0.14, ['10-13']=0.09}, true, nil)
animWalkRight	= anim8.newAnimation('loop', g('1-13,2'), 0.1, {[4]=0.12,[5]=0.14,[6]=0.14,[7]=0.14, ['10-13']=0.09})
animStandLeft	= anim8.newAnimation('once', g('1,1'), 0.5, nil, true)
animStandRight	= anim8.newAnimation('once', g('1,1'), 0.5)
animJumpUPLeft	= anim8.newAnimation('once', g('1,3'), 0.1, nil, true)
animJumpUPRight	= anim8.newAnimation('once', g('1,3'), 0.1)
animJumpDOWNLeft	= anim8.newAnimation('once', g('2,3'), 0.1, nil, true)
animJumpDOWNRight	= anim8.newAnimation('once', g('2,3'), 0.1)
animWalkUpstairsRight	= anim8.newAnimation('loop', g('1-10,4'), 0.08)
animWalkUpstairsLeft	= anim8.newAnimation('loop', g('1-10,4'), 0.08, nil, true)
animWalkDownstairsRight	= anim8.newAnimation('loop', g('1-8,5'), 0.08)
animWalkDownstairsLeft	= anim8.newAnimation('loop', g('1-8,5'), 0.08, nil, true)
animCrouchLeft	= anim8.newAnimation('once', g('1,6'), 0.5, nil, true)
animCrouchRight	= anim8.newAnimation('once', g('1,6'), 0.5)

local k = anim8.newGrid(174, 64, playerImage2:getWidth(), playerImage2:getHeight())
animAttackStandLeft		= anim8.newAnimation('loop', k('1-3,1'), 1, nil, true)
animAttackStandRight	= anim8.newAnimation('loop', k('1-3,1'), 1)
animAttackJumpLeft		= anim8.newAnimation('loop', k('1-3,2'), 1, nil, true)
animAttackJumpRight		= anim8.newAnimation('loop', k('1-3,2'), 1)
animAttackUpstairsLeft		= anim8.newAnimation('loop', k('1-3,5'), 1, nil, true)
animAttackUpstairsRight		= anim8.newAnimation('loop', k('1-3,5'), 1)
animAttackDownstairsLeft	= anim8.newAnimation('loop', k('1-3,4'), 1, nil, true)
animAttackDownstairsRight	= anim8.newAnimation('loop', k('1-3,4'), 1)
animAttackCrouchLeft		= anim8.newAnimation('loop', k('1-3,3'), 1, nil, true)
animAttackCrouchRight		= anim8.newAnimation('loop', k('1-3,3'), 1)

playerAnimation = animStandRight

function Player:new()
	local object = {
	x = 0,
	y = 0,
	width = 0,
	height = 0,
	xVelocity = 0,
	yVelocity = 0,
	xMaxVelocity = 800,
	yMaxVelocity = 400,
	walkSpeed = 0,
	jumpSpeed = 0,
	isJumping = false,
	isAttacking = false,
	attackTimer = 0,
	onFloor = false,
	hasJumped = false,
	direction = "right",
	onStairs = false,
	state = "",
	stairsPosition = 0,
	distanceFromStairs = 0,
	dialogState = nil,
	hitbox = {}
	}
	setmetatable( object, {__index = Player})
	return object
end

-- Movement functions
function Player:moveRight()
	self.xVelocity = self.walkSpeed
	playerAnimation:resume()
end

function Player:moveLeft()
	self.xVelocity = -1 * self.walkSpeed
	playerAnimation:resume()
end

function Player:jump()
	if self.onFloor then
		self.yVelocity = self.jumpSpeed
		self.onFloor = false
		self.isJumping = true
		self.hasJumped = true
		playerAnimation:resume()
	end
end

function Player:stop()
	self.xVelocity = 0
end

function Player:moveOnStairs(vertical, horizontal)
	self.onFloor = false
	self.onStairs = true
	if horizontal == 1 then
		self.xVelocity = self.walkSpeed
	else
		self.xVelocity = self.walkSpeed * -1
	end
	if vertical == "up" then
		self.yVelocity = self.walkSpeed * -1
	else
		self.yVelocity = self.walkSpeed
	end
end

function Player:collide(event)
	if event == "hitFloor" then
		self.yVelocity = 0
		self.onFloor = true
		if self.isJumping then self:stop() end
		self.isJumping = false
		animJumpUPRight:gotoFrame(1)
		animJumpUPRight.status = "playing"
		animJumpUPLeft:gotoFrame(1)
		animJumpUPLeft.status = "playing"
		animJumpDOWNRight:gotoFrame(1)
		animJumpDOWNRight.status = "playing"
		animJumpDOWNLeft:gotoFrame(1)
		animJumpDOWNLeft.status = "playing"
	end
	if event == "hitCeiling" then
		self.yVelocity = 0
	end
end

-- Update player
function Player:update(dt, gravity, map)
	if dt > 0.05 then dt = 0.005 end
	if love.keyboard.isDown( "d" )
		and self.onFloor
		and self.dialogState ~= "active"
		and self.state ~= "crouchRight"
		and self.state ~= "crouchLeft"
		and not self.isAttacking then
		self:moveRight()
	elseif love.keyboard.isDown( "d" )
		and love.keyboard.isDown( "s" )
		and self.onFloor
		and self.dialogState ~= "active"
		and self.state == "crouchLeft"
		and not self.isAttacking then
			self.direction = "right"
	end

	if love.keyboard.isDown( "a" )
		and self.onFloor
		and self.dialogState ~= "active"
		and self.state ~= "crouchRight"
		and self.state ~= "crouchLeft"
		and not self.isAttacking then
		self:moveLeft()
	elseif love.keyboard.isDown( "a" )
		and love.keyboard.isDown( "s" )
		and self.onFloor
		and self.dialogState ~= "active"
		and self.state == "crouchRight"
		and not self.isAttacking then
			self.direction = "left"
	end

	if love.keyboard.isDown( "w" ) and (self.onFloor or self.onStairs) and not love.keyboard.isDown( "s" ) and not self.isAttacking then
		-- Find stairs!
		local order = self:findStairs(map, self.x, self.y + 31, "UP")
		self.stairsPosition = order
		if order then
			if order == 1 then
				self:moveLeft()
			end
			if order == 4 then
				self:moveRight()
			end
			if order == 3 then
				self:moveOnStairs("up", 1)
				self.x = self.x + (self.xVelocity * dt)
				self.y = self.y + (self.yVelocity * dt)
				playerAnimation:resume()
				self.direction = "right"
			end
			if order == 2 then
				self:moveOnStairs("up", 0)
				self.x = self.x + (self.xVelocity * dt)
				self.y = self.y + (self.yVelocity * dt)
				playerAnimation:resume()
				self.direction = "left"
			end
		end
		if order == 0 then
			if love.keyboard.isDown( "w" ) and not (love.keyboard.isDown( "a" ) or love.keyboard.isDown( "d" )) then self:stop() end
			self.onFloor = true
			self.onStairs = false
			self.yVelocity = 0
		end
	end
	if love.keyboard.isDown( "s" ) and (self.onFloor or self.onStairs) and not love.keyboard.isDown( "w" ) and not self.isAttacking then
		-- Find stairs!
		local order = self:findStairs(map, self.x, self.y + 32, "DOWN")
		self.stairsPosition = order
		if order then
			if order == 1 then
				self:moveLeft()
			end
			if order == 4 then
				self:moveRight()
			end
			if order == 5 then
				self:moveOnStairs("down", 1)
				self.x = self.x + (self.xVelocity * dt)
				self.y = self.y + (self.yVelocity * dt)
				playerAnimation:resume()
				self.direction = "right"
			end
			if order == 6 then
				self:moveOnStairs("down", 0)
				self.x = self.x + (self.xVelocity * dt)
				self.y = self.y + (self.yVelocity * dt)
				playerAnimation:resume()
				self.direction = "left"
			end
		end
		if order == 0 then
			if love.keyboard.isDown( "s" ) and not (love.keyboard.isDown( "a" ) or love.keyboard.isDown( "d" )) then self:stop() end
			self.onFloor = true
			self.onStairs = false
			self.yVelocity = 0
		end
	end

	playerAnimation:update(dt)

	-- apply gravity when not on stairs
	if not(self.onStairs) then self.yVelocity = self.yVelocity + (gravity * dt) end
	-- speed limit
	self.xVelocity = math.clamp(self.xVelocity, -self.xMaxVelocity, self.xMaxVelocity)
	self.yVelocity = math.clamp(self.yVelocity, -self.yMaxVelocity, self.yMaxVelocity)

	local halfX = self.width / 2
	local halfY = self.height / 2
	-- move player
	local nextY = self.y + (self.yVelocity * dt)
	if self.yVelocity > 0 and not (self.onStairs) then -- going down
		if not (self:isColliding(map, self.x - halfX + 8, nextY + halfY))				-- bottom left
			and not (self:isColliding(map, self.x, nextY + halfY))						-- bottom middle
			and not (self:isColliding(map, self.x + halfX - 10, nextY + halfY)) then	-- bottom right
			self.y = nextY
			-- falling cancels X movement
			if self.onFloor then
				self.xVelocity = 0 
				self.yVelocity = 100
			end
			self.onFloor = false
		else
			self.y = math.round(nextY - ((nextY + 16) % map.tileHeight), 0.5) -- Add the tile width NOT halfY
			self:collide("hitFloor")
		end
	elseif self.yVelocity < 0 and not (self.onStairs) then -- going up
		if not (self:isColliding(map, self.x - halfX + 8, nextY - halfY - 1))				-- top left
			and not (self:isColliding(map, self.x, nextY - halfY - 1))						-- top middle
			and not (self:isColliding(map, self.x + halfX - 10, nextY - halfY - 1)) then	-- top right
			self.y = nextY
			self.onFloor = false
		else
			self.y = nextY + map.tileHeight - (nextY % map.tileHeight)
			self:collide("hitCeiling")
		end
	end

	local nextX = self.x + (self.xVelocity * dt)
	if self.xVelocity < 0 and not (self.onStairs) then
		if not (self:isColliding(map, nextX - halfX + 8, self.y + halfY - 1))		-- left bottom
			and not (self:isColliding(map, nextX - halfX + 8, self.y - halfY))		-- left top
			and not (self:isColliding(map, nextX - halfX + 8, self.y))				-- left middle 
			and not (self:isColliding(map, nextX - halfX + 8, self.y + 24))			-- left 1/4
			and not (self:isColliding(map, nextX - halfX + 8, self.y - 24))			-- left 1/4
			and not (self:isColliding(map, nextX - halfX + 8, self.y + 16))			-- left 2/4
			and not (self:isColliding(map, nextX - halfX + 8, self.y - 16))			-- left 2/4
			and not (self:isColliding(map, nextX - halfX + 8, self.y + 8 ))			-- left 3/4
			and not (self:isColliding(map, nextX - halfX + 8, self.y - 8 )) then	-- left 3/4
			self.x = self.x + (self.xVelocity * dt)
		else
			self.x = nextX + halfX - (nextX % map.tileWidth) - 8
		end
	elseif self.xVelocity > 0 and not (self.onStairs) then
		if not (self:isColliding(map, nextX + halfX - 9, self.y + halfY - 1))		-- left bottom
			and not (self:isColliding(map, nextX + halfX - 9, self.y - halfY))		-- left top
			and not (self:isColliding(map, nextX + halfX - 9, self.y))				-- left middle 
			and not (self:isColliding(map, nextX + halfX - 9, self.y + 24))			-- left 1/4
			and not (self:isColliding(map, nextX + halfX - 9, self.y - 24))			-- left 1/4
			and not (self:isColliding(map, nextX + halfX - 9, self.y + 16))			-- left 2/4
			and not (self:isColliding(map, nextX + halfX - 9, self.y - 16))			-- left 2/4
			and not (self:isColliding(map, nextX + halfX - 9, self.y + 8 ))			-- left 3/4
			and not (self:isColliding(map, nextX + halfX - 9, self.y - 8 )) then	-- left 3/4
			self.x = self.x + (self.xVelocity * dt)
		else
			self.x = nextX - halfX + (map.tileWidth - (nextX % map.tileWidth)) + 9
		end
	end

	-- update attack timer/animation/stuff
	if self.isAttacking then
		self.attackTimer = self.attackTimer - dt
		if self.attackTimer <= 0 then
			self.attackTimer = 0
			self.isAttacking = false
		end
	end

	-- update hitbox of the whip_on
	if self.isAttacking and self.attackTimer <= 0.3 then
		local s, hitbox, offset = 0, {}, 0
		-- s is direction | hitbox is array for coords | offset is for start point of hitbox
		if self.direction == "right" then s = 1 end
		if self.direction == "left" then s = -1 end
		if self.state == "stand" then offset = 18 end
		if self.state == "jump" or self.state == "fall" then offset = 22 end
		if self.state == "crouchRight" or self.state == "crouchLeft" then offset = 4 end
		if self.state == "moveStairs" then offset = 20 end
		hitbox.x1 = player.x
		hitbox.y1 = player.y - offset
		hitbox.x2 = player.x + (87 * s)
		hitbox.y2 = player.y - offset + 7
		self.hitbox = { hitbox.x1, hitbox.y1, hitbox.x2, hitbox.y2 }
	else
		self.hitbox = {}
	end

	-- update player state
	self.state = self:getState()
end

function Player:drawPlayer()
	if self.isAttacking then
		playerAnimation:draw(playerImage2, math.floor(self.x) - (174 / 2), math.floor(self.y) - (64 / 2))
	else
		playerAnimation:draw(playerImage, math.floor(self.x) - (32 / 2), math.floor(self.y) - (64 / 2))
	end
	--love.graphics.rectangle("line", self.x - (32 / 2) + 8, self.y - (64 / 2) + 2, self.width - 16, self.height)
end

function Player:spawnPlayer(x, y)
		self.x = x
		self.y = y
		self.onFloor = true
		self.onStairs = false
		camX = 1000
		camY = 100
end

function Player:isColliding(map, x, y)
	local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
	local tile = map("walls")(tileX, tileY)

	return not(tile == nil)
end

function Player:findStairs(map, x, y, search)
	-- seting variables
	local playerTile, direction, offset, distance, emptyTile = math.floor(self.x / map.tileWidth), 0, 0, 0, 0
	if search == "UP" then
		-- checking first tile [x][ ][ ]
		local tileX, tileY = math.floor((x - 16) / map.tileWidth), math.floor(y / map.tileHeight)
		local tile = map("stairs")(tileX, tileY)
		if not (tile == nil) then
			if tile.properties.direction == "right" then
				if self.onStairs then	return 3 end -- moveUpRight
				offset		= 0
				direction	= 1
			else
				if self.onStairs then	return 2 end -- moveUpLeft
				offset		= -14
				direction	= 0
			end
			distance = self.x - ((playerTile - 1) * 16) + offset
			if distance < -1 then		return 4 end -- moveRight
			if distance > 1 then		return 1 end -- moveLeft
			if (distance <= 1) and (distance >= -1) then
				if direction then		return 3	 -- moveUpRight
				else					return 2 end -- moveUpLeft
			end
		else
			emptyTile = emptyTile + 1
		end
		-- checking second tile [ ][x][ ]
		local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
		local tile = map("stairs")(tileX, tileY)
		if not (tile == nil) then
			if tile.properties.direction == "right" then
				if self.onStairs then	return 3 end -- moveUpRight
				offset		= 0
				direction	= 1
			else
				if self.onStairs then	return 2 end -- moveUpLeft
				offset		= -14
				direction	= 0
			end
			distance = self.x - (playerTile * 16) + offset
			if distance < -1 then		return 4 end -- moveRight
			if distance > 1 then		return 1 end -- moveLeft
			if (distance <= 1) and (distance >= -1) then
				if direction then		return 3	 -- moveUpRight
				else					return 2 end -- moveUpLeft
			end
		else
			emptyTile = emptyTile + 1
		end
		-- checking third tile [ ][ ][x]
		local tileX, tileY = math.floor((x+16) / map.tileWidth), math.floor(y / map.tileHeight)
		local tile = map("stairs")(tileX, tileY)
		if not (tile == nil) then
			if tile.properties.direction == "right" then
				if self.onStairs then	return 3 end -- moveUpRight
				offset		= 0
				direction	= 1
			else
				if self.onStairs then	return 2 end -- moveUpLeft
				offset		= -14
				direction	= 0
			end
			distance = self.x - ((playerTile + 1) * 16) + offset
			if distance < -1 then		return 4 end -- moveRight
			if distance > 1 then		return 1 end -- moveLeft
			if (distance <= 1) and (distance >= -1) then
				if direction then		return 3	 -- moveUpRight
				else					return 2 end -- moveUpLeft
			end
		else
			emptyTile = emptyTile + 1
		end
		if emptyTile == 3 then return 0 end
	end

--[[	  2    3
		   \  /
		1 - 00 - 4
		   /  \
		  6    5	]]--

	if search == "DOWN" then
		-- checking first tile [x][ ][ ]
		local tileX, tileY = math.floor((x - 16) / map.tileWidth), math.floor(y / map.tileHeight)
		local tile = map("stairs")(tileX, tileY)
		if not (tile == nil) then
			if tile.properties.direction == "right" then
				if self.onStairs then	return 6 end -- moveDownLeft
				offset		= -18
				direction	= 1
			else
				if self.onStairs then	return 5 end -- moveDownRight
				offset		= 0
				direction	= 0
			end
			distance = self.x - ((playerTile - 1) * 16) + offset
			if distance < -1 then		return 4 end -- moveRight
			if distance > 1 then		return 1 end -- moveLeft
			if (distance < 1) and (distance > -1) then
				if direction then		return 6	 -- moveDownLeft
				else					return 5 end -- moveDownRight
			end
		else
			emptyTile = emptyTile + 1
		end
		-- checking second tile [ ][x][ ]
		local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
		local tile = map("stairs")(tileX, tileY)
		if not (tile == nil) then
			if tile.properties.direction == "right" then
				if self.onStairs then	return 6 end -- moveDownLeft
				offset		= -18
				direction	= 1
			else
				if self.onStairs then	return 5 end -- moveDownRight
				offset		= 0
				direction	= 0
			end
			distance = self.x - (playerTile * 16) + offset
			if distance < -1 then		return 4 end -- moveRight
			if distance > 1 then		return 1 end -- moveLeft
			if (distance < 1) and (distance > -1) then
				if direction then		return 6	 -- moveDownLeft
				else					return 5 end -- moveDownRight
			end
		else
			emptyTile = emptyTile + 1
		end
		-- checking third tile [ ][ ][x]
		local tileX, tileY = math.floor((x+16) / map.tileWidth), math.floor(y / map.tileHeight)
		local tile = map("stairs")(tileX, tileY)
		if not (tile == nil) then
			if tile.properties.direction == "right" then
				if self.onStairs then	return 6 end -- moveDownLeft
				offset		= -18
				direction	= 1
			else
				if self.onStairs then	return 5 end -- moveDownRight
				offset		= 0
				direction	= 0
			end
			distance = self.x - ((playerTile + 1) * 16) + offset
			if distance < -1 then		return 4 end -- moveRight
			if distance > 1 then		return 1 end -- moveLeft
			if (distance < 1) and (distance > -1) then
				if direction then		return 6	 -- moveDownLeft
				else					return 5 end -- moveDownRight
			end
		else
			emptyTile = emptyTile + 1
		end
		if emptyTile == 3 then return 0 end
	end
end

function Player:getState()
	--[[ jump/fall walkRight walkLeft stand crouch ]]--
	local state = ""
	if self.onFloor then
		if self.xVelocity > 0 then
			state = "moveRight"
			self.direction = "right"
			playerAnimation = animWalkRight
		elseif self.xVelocity < 0 then
			state = "moveLeft"
			self.direction = "left"
			playerAnimation = animWalkLeft
		else
			if self.direction == "right" then
				-- if not crouching and attacking AND not pressing S
				if not (self.state == "crouchRight" and self.isAttacking) and not (love.keyboard.isDown( "s" ) and not self.isAttacking) then
					state = "stand"
					if self.isAttacking then
						playerAnimation = animAttackStandRight
						if self.attackTimer <= 0.5 then animAttackStandRight:gotoFrame(1) end
						if self.attackTimer <= 0.4 then animAttackStandRight:gotoFrame(2) end
						if self.attackTimer <= 0.3 then animAttackStandRight:gotoFrame(3) end
					else
						playerAnimation = animStandRight
					end
				end
				-- if you're pressing S but you're not attacking standing or attacking and crouching
				if (love.keyboard.isDown( "s" ) and not (self.isAttacking and self.state == "stand")) or (self.isAttacking and self.state == "crouchRight") then
					state = "crouchRight"
					if self.isAttacking then
						playerAnimation = animAttackCrouchRight
						if self.attackTimer <= 0.5 then animAttackCrouchRight:gotoFrame(1) end
						if self.attackTimer <= 0.4 then animAttackCrouchRight:gotoFrame(2) end
						if self.attackTimer <= 0.3 then animAttackCrouchRight:gotoFrame(3) end
					else
						playerAnimation = animCrouchRight
					end
				end
			else -- if direction is left
				-- if not crouching and attacking AND not pressing S
				if not (self.state == "crouchLeft" and self.isAttacking) and not (love.keyboard.isDown( "s" ) and not self.isAttacking) then
					state = "stand"
					if self.isAttacking then
						playerAnimation = animAttackStandLeft
						if self.attackTimer <= 0.5 then animAttackStandLeft:gotoFrame(1) end
						if self.attackTimer <= 0.4 then animAttackStandLeft:gotoFrame(2) end
						if self.attackTimer <= 0.3 then animAttackStandLeft:gotoFrame(3) end
					else
						playerAnimation = animStandLeft
					end
				end
				-- if you're pressing S but you're not attacking standing or attacking and crouching
				if (love.keyboard.isDown( "s" ) and not (self.isAttacking and self.state == "stand")) or (self.isAttacking and self.state == "crouchLeft") then
					state = "crouchLeft"
					if self.isAttacking then
						playerAnimation = animAttackCrouchLeft
						if self.attackTimer <= 0.5 then animAttackCrouchLeft:gotoFrame(1) end
						if self.attackTimer <= 0.4 then animAttackCrouchLeft:gotoFrame(2) end
						if self.attackTimer <= 0.3 then animAttackCrouchLeft:gotoFrame(3) end
					else
						playerAnimation = animCrouchLeft
					end
				end
			end
		end
	end
	if self.yVelocity < 0 or self.yVelocity > 0 then
		if self.yVelocity < 0 then
			if self.direction == "right" then
				if self.isAttacking then
					playerAnimation = animAttackJumpRight
					if self.attackTimer <= 0.5 then animAttackJumpRight:gotoFrame(1) end
					if self.attackTimer <= 0.4 then animAttackJumpRight:gotoFrame(2) end
					if self.attackTimer <= 0.3 then animAttackJumpRight:gotoFrame(3) end
				else
					playerAnimation = animJumpUPRight
				end
			else
				if self.isAttacking then
					playerAnimation = animAttackJumpLeft
					if self.attackTimer <= 0.5 then animAttackJumpLeft:gotoFrame(1) end
					if self.attackTimer <= 0.4 then animAttackJumpLeft:gotoFrame(2) end
					if self.attackTimer <= 0.3 then animAttackJumpLeft:gotoFrame(3) end
				else
					playerAnimation = animJumpUPLeft
				end
			end
			state = "jump"
		elseif self.yVelocity > 0 then
			if self.direction == "right" then
				if self.isAttacking then
					playerAnimation = animAttackJumpRight
					if self.attackTimer <= 0.5 then animAttackJumpRight:gotoFrame(1) end
					if self.attackTimer <= 0.4 then animAttackJumpRight:gotoFrame(2) end
					if self.attackTimer <= 0.3 then animAttackJumpRight:gotoFrame(3) end
				else
					playerAnimation = animJumpDOWNRight
				end
			else
				if self.isAttacking then
					playerAnimation = animAttackJumpLeft
					if self.attackTimer <= 0.5 then animAttackJumpLeft:gotoFrame(1) end
					if self.attackTimer <= 0.4 then animAttackJumpLeft:gotoFrame(2) end
					if self.attackTimer <= 0.3 then animAttackJumpLeft:gotoFrame(3) end
				else
					playerAnimation = animJumpDOWNLeft
				end
			end
			state = "fall"
		end
	end
	if self.onStairs then
		state = "moveStairs"
		if self.xVelocity > 0 and self.yVelocity < 0 then
			if self.isAttacking then
				playerAnimation = animAttackUpstairsRight
				if self.attackTimer <= 0.5 then animAttackUpstairsRight:gotoFrame(1) end
				if self.attackTimer <= 0.4 then animAttackUpstairsRight:gotoFrame(2) end
				if self.attackTimer <= 0.3 then animAttackUpstairsRight:gotoFrame(3) end
			else
				playerAnimation = animWalkUpstairsRight
			end
		elseif self.xVelocity < 0 and self.yVelocity < 0 then
			if self.isAttacking then
				playerAnimation = animAttackUpstairsLeft
				if self.attackTimer <= 0.5 then animAttackUpstairsLeft:gotoFrame(1) end
				if self.attackTimer <= 0.4 then animAttackUpstairsLeft:gotoFrame(2) end
				if self.attackTimer <= 0.3 then animAttackUpstairsLeft:gotoFrame(3) end
			else
				playerAnimation = animWalkUpstairsLeft
			end
		elseif self.xVelocity > 0 and self.yVelocity > 0 then
			if self.isAttacking then
				playerAnimation = animAttackDownstairsRight
				if self.attackTimer <= 0.5 then animAttackDownstairsRight:gotoFrame(1) end
				if self.attackTimer <= 0.4 then animAttackDownstairsRight:gotoFrame(2) end
				if self.attackTimer <= 0.3 then animAttackDownstairsRight:gotoFrame(3) end
			else
				playerAnimation = animWalkDownstairsRight
			end
		elseif self.xVelocity < 0 and self.yVelocity > 0 then
			if self.isAttacking then
				playerAnimation = animAttackDownstairsLeft
				if self.attackTimer <= 0.5 then animAttackDownstairsLeft:gotoFrame(1) end
				if self.attackTimer <= 0.4 then animAttackDownstairsLeft:gotoFrame(2) end
				if self.attackTimer <= 0.3 then animAttackDownstairsLeft:gotoFrame(3) end
			else
				playerAnimation = animWalkDownstairsLeft
			end
		end
	end

	return state
end

function Player:keypressed(key, unicode)
	if key == "p" and self.dialogState ~= "active" and not self.isAttacking then
		self:jump()
	end
	if key == "o"
		and not self.isAttacking
		and self.dialogState == nil
		and self.attackTimer == 0 then -- do wyjebania proly, jeśli nie jest zero to isAttacking = true ???
		if self.onFloor then self:stop() end
		playerAnimation:pause()
		self.isAttacking = true
		self.attackTimer = 0.5
	end
end